import Flutter
import UIKit
import MRZScannerSDK

public class SwiftMrzflutterpluginPlugin: NSObject, FlutterPlugin, MRZScannerDelegate {
    
    var _mrzScannerController : MRZScannerController?
    var pendingResult: FlutterResult?
    var continuousScanning = false
    static var channel: FlutterMethodChannel?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        channel = FlutterMethodChannel(name: "mrzflutterplugin", binaryMessenger: registrar.messenger())
        let instance = SwiftMrzflutterpluginPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel!)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "startScanner":
            pendingResult = result
            continuousScanning = false
            startScanner(scannerType: TYPE_MRZ,
                         partialRect: CGRect.init(x: 0, y: 0, width: 100, height: 100),
                         continuousScanning: false,
                         ignoreDuplicates: true)
            break
        case "startContinuousScanner":
            continuousScanning = true
            var ignoreDuplicates = true

            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    ignoreDuplicates = boolVal
                }
            }
            
            startScanner(scannerType: TYPE_MRZ,
                         partialRect: CGRect.init(x: 0, y: 0, width: 100, height: 100),
                         continuousScanning: true,
                         ignoreDuplicates: ignoreDuplicates)
            break
        case "startPartialViewScanner":
            pendingResult = result
            continuousScanning = false
            
            let arguments = call.arguments as! NSArray
            let x = CGFloat.init(exactly: arguments.object(at: 0) as! NSNumber)!
            let y = CGFloat.init(exactly: arguments.object(at: 1) as! NSNumber)!
            let width = CGFloat.init(exactly: arguments.object(at: 2) as! NSNumber)!
            let height = CGFloat.init(exactly: arguments.object(at: 3) as! NSNumber)!
            
            startScanner(scannerType: TYPE_MRZ,
                         partialRect: CGRect.init(x: x, y: y, width: width, height: height),
                         continuousScanning: false,
                         ignoreDuplicates: true)
            break
        case "captureIdImage":
            pendingResult = result
            continuousScanning = false
            
            startScanner(scannerType: TYPE_DOCUMENT_IMAGE_ID,
                         partialRect: CGRect.init(x: 0, y: 0, width: 100, height: 100),
                         continuousScanning: false,
                         ignoreDuplicates: true)
            break
        case "resumeScanner":
            if _mrzScannerController != nil {
                _mrzScannerController?.resumeScanner()
            }
            break
        case "closeScanner":
            if _mrzScannerController != nil {
                _mrzScannerController?.closeScanner()
            }
            break
        case "setNightModeActive":
            if _mrzScannerController != nil && call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    _mrzScannerController?.setNightModeActive(boolVal)
                }
            }
            break
        case "getSdkVersion":
            result(MRZScannerController.getSDKVersion())
            break
        case "registerWithLicenceKey":
            if call.arguments != nil {
                if let str = call.arguments as? String {
                    MRZScannerController.registerLicense(withKey: str)
                }
            }
            break
        case "scanFromGallery":
            pendingResult = result
            continuousScanning = false
            
            scanFromGallery()
            break
        case "setDateFormat":
            if call.arguments != nil {
                if let str = call.arguments as? String {
                    MRZScannerController.setDateFormat(str)
                }
            }
            break
        case "setPassportActive":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.setPassportActive(boolVal)
                }
            }
            break
        case "setIDActive":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.setIDActive(boolVal)
                }
            }
            break
        case "setVisaActive":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.setVisaActive(boolVal)
                }
            }
            break
        case "setVibrateOnSuccessfulScan":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.enableVibration(onSuccess: boolVal)
                }
            }
            break
        case "setUseFrontCamera":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.setUseFrontCamera(boolVal)
                }
            }
            break
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    func startScanner(scannerType: MRZScannerType, partialRect: CGRect, continuousScanning: Bool, ignoreDuplicates: Bool){
        let currentVC = UIApplication.shared.keyWindow?.rootViewController
        _mrzScannerController = MRZScannerController()
        _mrzScannerController!.delegate = self
        _mrzScannerController?.setScannerType(scannerType)
        _mrzScannerController?.setContinuousScanningEnabled(continuousScanning)
        _mrzScannerController?.setIgnoreDuplicates(ignoreDuplicates)
        currentVC?.addChild(_mrzScannerController!)
        _mrzScannerController?.initUI(currentVC, partialViewRect: partialRect)
    }
    
    func scanFromGallery(){
        let currentVC = UIApplication.shared.keyWindow?.rootViewController
        MRZScannerController.scan(fromGallery: currentVC, delegate: self)
    }
    
    public func successfulScan(withResult result: MRZResultDataModel!) {
        let jsonResult = result.toJSON()
        if result.portrait != nil
            || result.signature != nil
            || result.fullImage != nil
            || result.idBack != nil
            || result.idFront != nil
        {
            let data = jsonResult?.data(using: .utf8)
            
            do {
                var jsonDictionary = try JSONSerialization.jsonObject(with: data!, options:[]) as? Dictionary<String, Any?>
                
                if result.portrait != nil {
                    let imgData = result.portrait.pngData()
                    let base64Image = imgData?.base64EncodedData()
                    jsonDictionary?["portrait"] = String.init(data: base64Image!, encoding: .utf8)
                }
                
                if result.signature != nil {
                    let imgData = result.signature.pngData()
                    let base64Image = imgData?.base64EncodedData()
                    jsonDictionary?["signature"] = String.init(data: base64Image!, encoding: .utf8)
                }
                
                if result.fullImage != nil {
                    let imgData = result.fullImage.pngData()
                    let base64Image = imgData?.base64EncodedData()
                    jsonDictionary?["full_image"] = String.init(data: base64Image!, encoding: .utf8)
                }
                
                if result.idBack != nil {
                    let imgData = result.idBack.pngData()
                    let base64Image = imgData?.base64EncodedData()
                    jsonDictionary?["id_back"] = String.init(data: base64Image!, encoding: .utf8)
                }
                
                if result.idFront != nil {
                    let imgData = result.idFront.pngData()
                    let base64Image = imgData?.base64EncodedData()
                    jsonDictionary?["id_front"] = String.init(data: base64Image!, encoding: .utf8)
                }

                if let theJSONData = try? JSONSerialization.data(withJSONObject: jsonDictionary!,
                                                                 options: [])
                {
                    let theJSONText = String(data: theJSONData, encoding: .utf8)
                    if continuousScanning {
                        SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: theJSONText)
                    } else {
                        pendingResult?(theJSONText)
                    }
                } else {
                    if continuousScanning {
                        SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: jsonResult)
                    } else {
                        pendingResult?(jsonResult)
                    }
                }
            } catch {
                if continuousScanning {
                    SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: jsonResult)
                } else {
                    pendingResult?(jsonResult)
                }
            }
        } else {
            if continuousScanning {
                SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: jsonResult)
            } else {
                pendingResult?(jsonResult)
            }
        }
        
        if !continuousScanning {
            _mrzScannerController?.dismiss(animated: true, completion: nil)
        }
    }
    
    public func scannerWasDismissed() {
        if continuousScanning {
            SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: "Error:scannerWasDismissed")
        } else {
            pendingResult?(FlutterError.init(code: "scannerWasDismissed", message: "scannerWasDismissed", details: nil))
        }
    }
    
    public func permissionsWereDenied() {
        if continuousScanning {
            SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: "Error:permissionsWereDenied")
        } else {
            pendingResult?(FlutterError.init(code: "permissionsWereDenied", message: "permissionsWereDenied", details: nil))
        }
    }
    
    public func scanImageFailed() {
        if continuousScanning {
            SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: "Error:scanImageFailed")
        } else {
            pendingResult?(FlutterError.init(code: "scanImageFailed", message: "scanImageFailed", details: nil))
        }
    }
    
    public func successfulDocumentScan(withImageResult resultImage: UIImage!) {
        let imgData = resultImage.pngData()
        guard let base64Image = imgData?.base64EncodedData() else {
            if continuousScanning {
                SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: "Error:Conversion to base64 failed")
            } else {
                pendingResult?(FlutterError.init(code: "scanImageFailed", message: "Conversion to base64 failed", details: nil))
            }
            return
        }
        
        guard let imgString = String.init(data: base64Image, encoding: .utf8) else {
            if continuousScanning {
                SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: "Error:Conversion to base64 failed")
            } else {
                pendingResult?(FlutterError.init(code: "scanImageFailed", message: "Conversion to base64 failed", details: nil))
            }
            return
        }
        
        if continuousScanning {
            SwiftMrzflutterpluginPlugin.channel?.invokeMethod("onScannerResult", arguments: imgString)
        } else {
            pendingResult?(imgString)
        }
    }
    
    public func detachFromEngine(for registrar: FlutterPluginRegistrar) {
        SwiftMrzflutterpluginPlugin.channel = nil
        pendingResult = nil
    }
}
